@@ -10,8 +10,8 @@ TODO
 from __future__ import print_function, division, absolute_import
 import sys
 from os.path import splitext
-from cutadapt.xopen import xopen
-from cutadapt.compat import zip, basestring
+from .xopen import xopen
+from .compat import zip, basestring
 
 __author__ = "Marcel Martin"
 
@@ -34,33 +34,37 @@ def _shorten(s, n=100):
 class Sequence(object):
 	"""qualities is a string and it contains the qualities encoded as ascii(qual+33)."""
 
-	def __init__(self, name, sequence, qualities=None, name2='', match=None):
+	def __init__(self, name, sequence, qualities=None, name2='', match=None, match_info=None):
 		"""Set qualities to None if there are no quality values"""
 		self.name = name
 		self.sequence = sequence
 		self.qualities = qualities
 		self.name2 = name2
 		self.match = match
+		self.match_info = match_info
+		self.original_length = len(sequence)
 		if qualities is not None:
 			if len(qualities) != len(sequence):
 				rname = _shorten(name)
-				raise FormatError("In read named {0!r}: Length of quality sequence ({1}) and length of read ({2}) do not match".format(
-					rname, len(qualities), len(sequence)))
-
+				raise FormatError("In read named {0!r}: Length of quality sequence ({1}) and "
+					"length of read ({2}) do not match".format(rname, len(qualities), len(sequence)))
+	
 	def __getitem__(self, key):
 		"""slicing"""
 		return self.__class__(
 			self.name,
 			self.sequence[key],
 			self.qualities[key] if self.qualities is not None else None,
-		    self.name2,
-		    self.match)
+			self.name2,
+			self.match,
+			self.match_info)
 
 	def __repr__(self):
 		qstr = ''
 		if self.qualities is not None:
 			qstr = ', qualities={0!r}'.format(_shorten(self.qualities))
-		return '<Sequence(name={0!r}, sequence={1!r}{2})>'.format(_shorten(self.name), _shorten(self.sequence), qstr)
+		return '<Sequence(name={0!r}, sequence={1!r}{2})>'.format(
+			_shorten(self.name), _shorten(self.sequence), qstr)
 
 	def __len__(self):
 		return len(self.sequence)
@@ -74,6 +78,34 @@ class Sequence(object):
 		return not self.__eq__(other)
 
 
+class SequenceReader(object):
+	"""Read possibly compressed files containing sequences"""
+	_close_on_exit = False
+
+	def __init__(self, file):
+		"""
+		file is a path or a file-like object. In both cases, the file may
+		be compressed (.gz, .bz2, .xz).
+		"""
+		if isinstance(file, basestring):
+			file = xopen(file)
+			self._close_on_exit = True
+		self._file = file
+
+	def close(self):
+		if self._close_on_exit and self._file is not None:
+			self._file.close()
+			self._file = None
+
+	def __enter__(self):
+		if self._file is None:
+			raise ValueError("I/O operation on closed SequenceReader")
+		return self
+
+	def __exit__(self, *args):
+		self.close()
+
+
 try:
 	from ._seqio import Sequence
 except ImportError:
@@ -81,7 +113,7 @@ except ImportError:
 
 
 class ColorspaceSequence(Sequence):
-	def __init__(self, name, sequence, qualities, primer=None, name2='', match=None):
+	def __init__(self, name, sequence, qualities, primer=None, name2='', match=None, match_info=None):
 		# In colorspace, the first character is the last nucleotide of the primer base
 		# and the second character encodes the transition from the primer base to the
 		# first real base of the read.
@@ -95,7 +127,7 @@ class ColorspaceSequence(Sequence):
 			raise FormatError("In read named {0!r}: length of colorspace quality "
 				"sequence ({1}) and length of read ({2}) do not match (primer "
 				"is: {3!r})".format(rname, len(qualities), len(sequence), self.primer))
-		super(ColorspaceSequence, self).__init__(name, sequence, qualities, name2, match)
+		super(ColorspaceSequence, self).__init__(name, sequence, qualities, name2, match, match_info)
 		if not self.primer in ('A', 'C', 'G', 'T'):
 			raise FormatError("Primer base is {0!r} in read {1!r}, but it "
 				"should be one of A, C, G, T.".format(
@@ -105,7 +137,8 @@ class ColorspaceSequence(Sequence):
 		qstr = ''
 		if self.qualities is not None:
 			qstr = ', qualities={0!r}'.format(_shorten(self.qualities))
-		return '<ColorspaceSequence(name={0!r}, primer={1!r}, sequence={2!r}{3})>'.format(_shorten(self.name), self.primer, _shorten(self.sequence), qstr)
+		return '<ColorspaceSequence(name={0!r}, primer={1!r}, sequence={2!r}{3})>'.format(
+			_shorten(self.name), self.primer, _shorten(self.sequence), qstr)
 
 	def __getitem__(self, key):
 		return self.__class__(
@@ -114,7 +147,8 @@ class ColorspaceSequence(Sequence):
 			self.qualities[key] if self.qualities is not None else None,
 			self.primer,
 			self.name2,
-			self.match)
+			self.match,
+			self.match_info)
 
 
 def sra_colorspace_sequence(name, sequence, qualities, name2):
@@ -152,23 +186,18 @@ class FileWithPrependedLine(object):
 		self._file.close()
 
 
-class FastaReader(object):
+class FastaReader(SequenceReader):
 	"""
 	Reader for FASTA files.
 	"""
-	_close_on_exit = False
-
 	def __init__(self, file, keep_linebreaks=False, sequence_class=Sequence):
 		"""
-		file is a filename or a file-like object.
-		If file is a filename, then it is passed to xopen().
+		file is a path or a file-like object. In both cases, the file may
+		be compressed (.gz, .bz2, .xz).
 
 		keep_linebreaks -- whether to keep newline characters in the sequence
 		"""
-		if isinstance(file, basestring):
-			file = xopen(file)
-			self._close_on_exit = True
-		self._file = file
+		super(FastaReader, self).__init__(file)
 		self.sequence_class = sequence_class
 		self.delivers_qualities = False
 		self._delimiter = '\n' if keep_linebreaks else ''
@@ -200,43 +229,24 @@ class FastaReader(object):
 		if name is not None:
 			yield self.sequence_class(name, self._delimiter.join(seq), None)
 
-	def close(self):
-		if self._close_on_exit and self._file is not None:
-			self._file.close()
-			self._file = None
-
-	def __enter__(self):
-		if self._file is None:
-			raise ValueError("I/O operation on closed FastaReader")
-		return self
-
-	def __exit__(self, *args):
-		self.close()
-
 
 class ColorspaceFastaReader(FastaReader):
 	def __init__(self, file, keep_linebreaks=False):
 		super(ColorspaceFastaReader, self).__init__(file, keep_linebreaks, sequence_class=ColorspaceSequence)
 
 
-class FastqReader(object):
+class FastqReader(SequenceReader):
 	"""
 	Reader for FASTQ files. Does not support multi-line FASTQ files.
 	"""
-	_close_on_exit = False
-
 	def __init__(self, file, sequence_class=Sequence): # TODO could be a class attribute
 		"""
-		file is a filename or a file-like object.
-		If file is a filename, then .gz files are supported.
+		file is a path or a file-like object. compressed files are supported.
 
 		The sequence_class should be a class such as Sequence or
 		ColorspaceSequence.
 		"""
-		if isinstance(file, basestring):
-			file = xopen(file)
-			self._close_on_exit = True
-		self._file = file
+		super(FastqReader, self).__init__(file)
 		self.sequence_class = sequence_class
 		self.delivers_qualities = True
 
@@ -245,17 +255,20 @@ class FastqReader(object):
 		Return tuples: (name, sequence, qualities).
 		qualities is a string and it contains the unmodified, encoded qualities.
 		"""
+		i = 3
 		for i, line in enumerate(self._file):
 			if i % 4 == 0:
 				if not line.startswith('@'):
-					raise FormatError("Line {0} in FASTQ file is expected to start with '@', but found {1!r}".format(i+1, line[:10]))
+					raise FormatError("Line {0} in FASTQ file is expected to start with '@', "
+						"but found {1!r}".format(i+1, line[:10]))
 				name = line.strip()[1:]
 			elif i % 4 == 1:
 				sequence = line.strip()
 			elif i % 4 == 2:
 				line = line.strip()
 				if not line.startswith('+'):
-					raise FormatError("Line {0} in FASTQ file is expected to start with '+', but found {1!r}".format(i+1, line[:10]))
+					raise FormatError("Line {0} in FASTQ file is expected to start with '+', "
+						"but found {1!r}".format(i+1, line[:10]))
 				if len(line) > 1:
 					if line[1:] != name:
 						raise FormatError(
@@ -270,23 +283,12 @@ class FastqReader(object):
 			elif i % 4 == 3:
 				qualities = line.rstrip('\n\r')
 				yield self.sequence_class(name, sequence, qualities, name2=name2)
-
-	def close(self):
-		if self._close_on_exit and self._file is not None:
-			self._file.close()
-			self._file = None
-
-	def __enter__(self):
-		if self._file is None:
-			raise ValueError("I/O operation on closed FastqReader")
-		return self
-
-	def __exit__(self, *args):
-		self.close()
+		if i % 4 != 3:
+			raise FormatError("FASTQ file ended prematurely")
 
 
 try:
-	from ._seqio import FastqReader, FormatError
+	from ._seqio import FastqReader
 except ImportError:
 	pass
 
@@ -329,11 +331,13 @@ class FastaQualReader(object):
 			conv[str(i)] = chr(i + 33)
 		for fastaread, qualread in zip(self.fastareader, self.qualreader):
 			if fastaread.name != qualread.name:
-				raise FormatError("The read names in the FASTA and QUAL file do not match ({0!r} != {1!r})".format(fastaread.name, qualread.name))
+				raise FormatError("The read names in the FASTA and QUAL file "
+					"do not match ({0!r} != {1!r})".format(fastaread.name, qualread.name))
 			try:
 				qualities = ''.join([conv[value] for value in qualread.sequence.split()])
 			except KeyError as e:
-				raise FormatError("Within read named {0!r}: Found invalid quality value {1}".format(fastaread.name, e))
+				raise FormatError("Within read named {0!r}: Found invalid quality "
+					"value {1}".format(fastaread.name, e))
 			assert fastaread.name == qualread.name
 			yield self.sequence_class(fastaread.name, fastaread.sequence, qualities)
 
@@ -355,15 +359,16 @@ class ColorspaceFastaQualReader(FastaQualReader):
 
 def sequence_names_match(r1, r2):
 	"""
-	Check whether the sequences r1 and r2 have identical names (ignoring /1 and
-	/2 suffixes).
+	Check whether the sequences r1 and r2 have identical names, ignoring a
+	suffix of '1' or '2'. Some old paired-end reads have names that end in '/1'
+	and '/2'. Also, the fastq-dump tool (used for converting SRA files to FASTQ)
+	appends a .1 and .2 to paired-end reads if option -I is used.
 	"""
 	name1 = r1.name.split(None, 1)[0]
 	name2 = r2.name.split(None, 1)[0]
-	if name1[-2:-1] == '/':
-		name1 = name1[:-2]
-	if name2[-2:-1] == '/':
-		name2 = name2[:-2]
+	if name1[-1:] in '12' and name2[-1:] in '12':
+		name1 = name1[:-1]
+		name2 = name2[:-1]
 	return name1 == name2
 
 
@@ -392,14 +397,16 @@ class PairedSequenceReader(object):
 				# End of file 1. Make sure that file 2 is also at end.
 				try:
 					next(it2)
-					raise FormatError("Reads are improperly paired. There are more reads in file 2 than in file 1.")
+					raise FormatError("Reads are improperly paired. There are more reads in "
+						"file 2 than in file 1.")
 				except StopIteration:
 					pass
 				break
 			try:
 				r2 = next(it2)
 			except StopIteration:
-				raise FormatError("Reads are improperly paired. There are more reads in file 1 than in file 2.")
+				raise FormatError("Reads are improperly paired. There are more reads in "
+					"file 1 than in file 2.")
 			if not sequence_names_match(r1, r2):
 				raise FormatError("Reads are improperly paired. Read name '{0}' "
 					"in file 1 does not match '{1}' in file 2.".format(r1.name, r2.name))
@@ -446,24 +453,45 @@ class InterleavedSequenceReader(object):
 	def __exit__(self, *args):
 		self.close()
 
+class FileWriter(object):
+	def __init__(self, file):
+		if isinstance(file, str):
+			self._file = xopen(file, 'w')
+			self._close_on_exit = True
+		else:
+			self._file = file
+			self._close_on_exit = False
+	
+	def close(self):
+		if self._close_on_exit:
+			self._file.close()
+	
+	def __enter__(self):
+		if self._file.closed:
+			raise ValueError("I/O operation on closed file")
+		return self
 
-class FastaWriter(object):
+	def __exit__(self, *args):
+		self.close()
+
+class SingleRecordWriter(object):
+	"""Public interface to single-record files"""
+	def write(self, record):
+		raise NotImplementedError()
+
+class FastaWriter(FileWriter, SingleRecordWriter):
 	"""
 	Write FASTA-formatted sequences to a file.
 	"""
-	_close_on_exit = False
 
 	def __init__(self, file, line_length=None):
 		"""
 		If line_length is not None, the lines will
 		be wrapped after line_length characters.
 		"""
+		FileWriter.__init__(self, file)
 		self.line_length = line_length if line_length != 0 else None
-		if isinstance(file, str):
-			file = xopen(file, 'w')
-			self._close_on_exit = True
-		self._file = file
-
+	
 	def write(self, name_or_seq, sequence=None):
 		"""Write an entry to the the FASTA file.
 
@@ -482,6 +510,7 @@ class FastaWriter(object):
 			sequence = name_or_seq.sequence
 		else:
 			name = name_or_seq
+		
 		if self.line_length is not None:
 			print('>{0}'.format(name), file=self._file)
 			for i in range(0, len(sequence), self.line_length):
@@ -491,27 +520,13 @@ class FastaWriter(object):
 		else:
 			print('>{0}'.format(name), sequence, file=self._file, sep='\n')
 
-	def close(self):
-		if self._close_on_exit:
-			self._file.close()
-
-	def __enter__(self):
-		if self._file.closed:
-			raise ValueError("I/O operation on closed file")
-		return self
-
-	def __exit__(self, *args):
-		self.close()
-
-
 class ColorspaceFastaWriter(FastaWriter):
 	def write(self, record):
 		name = record.name
 		sequence = record.primer + record.sequence
 		super(ColorspaceFastaWriter, self).write(name, sequence)
 
-
-class FastqWriter(object):
+class FastqWriter(FileWriter, SingleRecordWriter):
 	"""
 	Write sequences with qualities in FASTQ format.
 
@@ -521,14 +536,6 @@ class FastqWriter(object):
 	+
 	QUALITIS
 	"""
-	_close_on_exit = False
-
-	def __init__(self, file):
-		if isinstance(file, str):
-			file = xopen(file, "w")
-			self._close_on_exit = True
-		self._file = file
-
 	def write(self, record):
 		"""
 		Write a Sequence record to the the FASTQ file.
@@ -543,19 +550,6 @@ class FastqWriter(object):
 		print("@{0:s}\n{1:s}\n+\n{2:s}".format(
 			name, sequence, qualities), file=self._file)
 
-	def close(self):
-		if self._close_on_exit:
-			self._file.close()
-
-	def __enter__(self):
-		if self._file.closed:
-			raise ValueError("I/O operation on closed file")
-		return self
-
-	def __exit__(self, *args):
-		self.close()
-
-
 class ColorspaceFastqWriter(FastqWriter):
 	def write(self, record):
 		name = record.name
@@ -563,11 +557,26 @@ class ColorspaceFastqWriter(FastqWriter):
 		qualities = record.qualities
 		super(ColorspaceFastqWriter, self).writeseq(name, sequence, qualities)
 
+class PairRecordWriter(object):
+	"""Public interface to paired-record files"""
+	def write(self, read1, read2):
+		raise NotImplementedError()
+	def close(self):
+		raise NotImplementedError()
+	
+	def __enter__(self):
+		# TODO do not allow this twice
+		return self
 
-class PairedSequenceWriter(object):
-	def __init__(self, file1, file2, colorspace=False, fileformat='fastq'):
-		self._writer1 = open(file1, colorspace=colorspace, fileformat=fileformat, mode='w')
-		self._writer2 = open(file2, colorspace=colorspace, fileformat=fileformat, mode='w')
+	def __exit__(self, *args):
+		self.close()
+
+class PairedSequenceWriter(PairRecordWriter):
+	def __init__(self, file1, file2, colorspace=False, fileformat='fastq', qualities=None):
+		self._writer1 = open(file1, colorspace=colorspace, fileformat=fileformat, mode='w',
+			qualities=qualities)
+		self._writer2 = open(file2, colorspace=colorspace, fileformat=fileformat, mode='w',
+			qualities=qualities)
 
 	def write(self, read1, read2):
 		self._writer1.write(read1)
@@ -577,20 +586,12 @@ class PairedSequenceWriter(object):
 		self._writer1.close()
 		self._writer2.close()
 
-	def __enter__(self):
-		# TODO do not allow this twice
-		return self
-
-	def __exit__(self, *args):
-		self.close()
-
-
-class InterleavedSequenceWriter(object):
+class InterleavedSequenceWriter(PairRecordWriter):
 	"""
 	Write paired-end reads to an interleaved FASTA or FASTQ file
 	"""
-	def __init__(self, file, colorspace=False, fileformat='fastq'):
-		self._writer = open(file, colorspace=colorspace, fileformat=fileformat, mode='w')
+	def __init__(self, file, colorspace=False, fileformat='fastq', qualities=None):
+		self._writer = open(file, colorspace=colorspace, fileformat=fileformat, mode='w', qualities=qualities)
 
 	def write(self, read1, read2):
 		self._writer.write(read1)
@@ -599,43 +600,45 @@ class InterleavedSequenceWriter(object):
 	def close(self):
 		self._writer.close()
 
-	def __enter__(self):
-		return self
-
-	def __exit__(self, *args):
-		self.close()
-
-
 class UnknownFileType(Exception):
 	"""
 	Raised when open could not autodetect the file type.
 	"""
 
 
-def open(file1, file2=None, qualfile=None,
-	colorspace=False, fileformat=None, interleaved=False, mode='r'):
+def open(file1, file2=None, qualfile=None, colorspace=False, fileformat=None,
+	interleaved=False, mode='r', qualities=None):
 	"""
-	Open sequence file in FASTA or FASTQ format. Parameters file1, file2 and
-	qualfile can be paths to regular or compressed files or file-like objects.
-	
-	If only file1 is provided and interleaved is True, an
-	InterleavedSequenceReader (for paired-end reads) is returned. If only file1 is provided and interleaved is False, a FastaReader
-	or FastqReader (for single-end reads) is returned. If file2 is also
-	provided, a PairedSequenceReader is returned. If qualfile is given, a
-	FastaQualReader from file1 and qualfile is returned. One of file2 and
-	qualfile must always be None (no paired-end data is supported when reading
-	qualfiles).
+	Open sequence files in FASTA or FASTQ format for reading or writing. This is
+	a factory that returns an instance of one of the ...Reader or ...Writer
+	classes also defined in this module.
 
-	If the colorspace parameter is set to True, the returned readers are
-	ColorspaceFastaReader, ColorspaceFastqReader or ColorspaceFastaQualReader
-	instead.
+	file1, file2, qualfile -- Paths to regular or compressed files or file-like
+		objects. Use file1 if data is single-end. If also file2 is provided,
+		sequences are paired. If qualfile is given, then file1 must be a FASTA
+		file and sequences are single-end. One of file2 and qualfile must always
+		be None (no paired-end data is supported when reading qualfiles).
 
-	If possible, file format is autodetected by inspecting the file name:
-	.fasta/.fa, .fastq/.fq and some other extensions are allowed. If the
-	file name is not available (when reading from standard input), the file is
-	read and the file type determined from the content. The autodetection can
-	be skipped by setting fileformat to one of 'fasta', 'fastq', 'sra-fastq'.
-	Colorspace is not auto-detected and must always be requested explicitly.
+	mode -- Either 'r' for reading or 'w' for writing.
+
+	interleaved -- If True, then file1 contains interleaved paired-end data.
+		file2 and qualfile must be None in this case.
+
+	colorspace -- If True, instances of the Colorspace... classes
+		are returned.
+
+	fileformat -- If set to None, file format is autodetected from the file name
+		extension. Set to 'fasta', 'fastq', or 'sra-fastq' to not auto-detect.
+		Colorspace is not auto-detected and must always be requested explicitly.
+
+	qualities -- When mode is 'w' and fileformat is None, this can be set to
+		True or False to specify whether the written sequences will have quality
+		values. This is is used in two ways:
+		* If the output format cannot be determined (unrecognized extension
+		  etc), no exception is raised, but fasta or fastq format is chosen
+		  appropriately.
+		* When False (no qualities available), an exception is raised when the
+		  auto-detected output format is FASTQ.
 	"""
 	if mode not in ('r', 'w'):
 		raise ValueError("Mode must be 'r' or 'w'")
@@ -647,13 +650,13 @@ def open(file1, file2=None, qualfile=None,
 		if mode == 'r':
 			return PairedSequenceReader(file1, file2, colorspace, fileformat)
 		else:
-			return PairedSequenceWriter(file1, file2, colorspace, fileformat)
+			return PairedSequenceWriter(file1, file2, colorspace, fileformat, qualities)
 
 	if interleaved:
 		if mode == 'r':
 			return InterleavedSequenceReader(file1, colorspace, fileformat)
 		else:
-			return InterleavedSequenceWriter(file1, colorspace, fileformat)
+			return InterleavedSequenceWriter(file1, colorspace, fileformat, qualities)
 
 	if qualfile is not None:
 		if mode == 'w':
@@ -663,65 +666,91 @@ def open(file1, file2=None, qualfile=None,
 			return ColorspaceFastaQualReader(file1, qualfile)
 		else:
 			return FastaQualReader(file1, qualfile)
-	# read from FASTA or FASTQ
+
+	# All the multi-file things have been dealt with, delegate rest to the
+	# single-file function.
+	return _seqopen1(file1, colorspace=colorspace, fileformat=fileformat,
+		mode=mode, qualities=qualities)
+
+
+def _seqopen1(file, colorspace=False, fileformat=None, mode='r', qualities=None):
+	"""
+	Open a single sequence file. See description above.
+	"""
 	if mode == 'r':
 		fastq_handler = ColorspaceFastqReader if colorspace else FastqReader
 		fasta_handler = ColorspaceFastaReader if colorspace else FastaReader
-	else:
+	elif mode == 'w':
 		fastq_handler = ColorspaceFastqWriter if colorspace else FastqWriter
 		fasta_handler = ColorspaceFastaWriter if colorspace else FastaWriter
+	else:
+		raise ValueError("Mode must be 'r' or 'w'")
 
-	if fileformat is not None:  # explict file format given
+	if fileformat:  # Explict file format given
 		fileformat = fileformat.lower()
 		if fileformat == 'fasta':
-			return fasta_handler(file1)
+			return fasta_handler(file)
 		elif fileformat == 'fastq':
-			return fastq_handler(file1)
+			return fastq_handler(file)
 		elif fileformat == 'sra-fastq' and colorspace:
 			if mode == 'w':
 				raise NotImplementedError('Writing to sra-fastq not supported')
-			return SRAColorspaceFastqReader(file1)
+			return SRAColorspaceFastqReader(file)
 		else:
 			raise UnknownFileType("File format {0!r} is unknown (expected "
 				"'sra-fastq' (only for colorspace), 'fasta' or 'fastq').".format(fileformat))
 
-	if mode == 'w':
-		raise NotImplementedError('autodetection of file type for output files not implemented')
-
-	# Try to detect the file format
+	# Detect file format
 	name = None
-	if file1 == "-":
-		file1 = sys.stdin
-	elif isinstance(file1, basestring):
-		name = file1
-	elif hasattr(file1, "name"):  # file1 seems to be an open file-like object
-		name = file1.name
+	if file == "-":
+		file = sys.stdin if mode == 'r' else sys.stdout
+	elif isinstance(file, basestring):
+		name = file
+	elif hasattr(file, "name"):  # seems to be an open file-like object
+		name = file.name
 
-	if name is not None:
-		if name.endswith('.gz'):
-			name = name[:-3]
-		elif name.endswith('.xz'):
-			name = name[:-3]
-		elif name.endswith('.bz2'):
-			name = name[:-4]
+	if name:
+		for ext in ('.gz', '.xz', '.bz2'):
+			if name.endswith(ext):
+				name = name[:-len(ext)]
+				break
 		name, ext = splitext(name)
 		ext = ext.lower()
 		if ext in ['.fasta', '.fa', '.fna', '.csfasta', '.csfa']:
-			return fasta_handler(file1)
+			format = 'fasta'
 		elif ext in ['.fastq', '.fq'] or (ext == '.txt' and name.endswith('_sequence')):
-			return fastq_handler(file1)
+			format = 'fastq'
+		elif mode == 'w' and qualities is True:
+			# Format not recognized, but know we want to write reads with qualities
+			format = 'fastq'
+		elif mode == 'w' and qualities is False:
+			# Same, but we know that we want to write reads without qualities
+			format = 'fasta'
 		else:
 			raise UnknownFileType("Could not determine whether file {0!r} is FASTA "
-				"or FASTQ: file name extension {1!r} not recognized".format(file1, ext))
+				"or FASTQ: file name extension {1!r} not recognized".format(file, ext))
+		if format == 'fastq' and qualities is False:
+			raise ValueError("Output format cannot be FASTQ since no quality "
+				"values are available.")
+		if format == 'fastq':
+			return fastq_handler(file)
+		else:
+			return fasta_handler(file)
 
-	# No name available.
-	# autodetect type by reading from the file
-	for line in file1:
+	if mode == 'w':
+		if qualities is True:
+			return fastq_handler(file)
+		elif qualities is False:
+			return fasta_handler(file)
+		raise UnknownFileType('Cannot determine whether to write in FASTA or '
+			'FASTQ format')
+	# No name available. Try to autodetect type by reading from the file.
+	for line in file:
 		if line.startswith('#'):
 			# Skip comment lines (needed for csfasta)
 			continue
 		if line.startswith('>'):
-			return fasta_handler(FileWithPrependedLine(file1, line))
+			return fasta_handler(FileWithPrependedLine(file, line))
 		if line.startswith('@'):
-			return fastq_handler(FileWithPrependedLine(file1, line))
+			return fastq_handler(FileWithPrependedLine(file, line))
 	raise UnknownFileType("File is neither FASTQ nor FASTA.")
